<?php

require_once("binary.php");

$filename = "../rom_u";

$f = file_get_contents($filename);
if ((strlen($f) & 0x7FFF) === 0x200) {
	// Remove header
	$f = substr($f,0x200);
}

function parse_seq_addresses($f){
	$sa=0xE9A7 + 0x8000;
	$ea=0xEAE7 + 0x8000;
	$entries=array();
	print "<h2>Sequence adresses</h2>";
	print "<table border='1'>\r\n";
	for($i=1,$p=$sa;$p<$ea;++$i){
		$ad=getint2($f,$p);
		$entries[$ad][]=$i;
		printf('<tr><td>%d = $%02X</td><td>%s</td></tr>',$i,$i,jump_addr($ad));
	}
	print "</table>\r\n";
	//print_r($entries);
	//die();
	return $entries;
}

function parse_frame_table($f){
	$frame_table_addrs = array(
		0x8000,
		0x85a0,
		0x870e,
		0x890c,
		0x8ae0,
		0x8b22,
		0x8e0a,
		0x9074,
		0x9134,
		0x91F4 // end
	);
	$frame_table_chtabs = array(
		0,
		0,
		0,
		8,
		-1,
		9,
		10,
		11,
		12,
	);
	$frame_table_names = array(
		"kid",
		"guard",
		"PV",
		"monster",
		"mouse",
		"Jaffar",
		"kid special",
		"Princess ending",
		"kid ending",
	);

	$chtab_bases = array(
		0x0000,
		0x0043,
		0x0086,
		0x00f4, // $01:bf84,X
		0x0194,
		0x01c1,
		0x0271,
		0x0228,
		0x03dc,
		0x032d,
		0x02bc,
		0x0279,
		0x0299,
		-1=>0x0435,
	);

	print "<h2>Frame tables</h2>";

	$frame_sprite = array();
	$n = count($frame_table_addrs) - 1;
	for ($which_table = 0; $which_table < $n; ++$which_table) {
		printf("TABLE %d (%s)\n", $which_table, $frame_table_names[$which_table]);
		$frame_table_start = $frame_table_addrs[$which_table] + 0x8000;
		$frame_table_end = $frame_table_addrs[$which_table + 1] + 0x8000;
		$frame_table_chtab = $frame_table_chtabs[$which_table];
		$frame = ($which_table===1 ? 0x95 : 0) + 1;
		print("<table border='1'>");
		for ($p = $frame_table_start; $p < $frame_table_end; ) {
			$data = getstr($f,$p,6);
			$image = ord($data[0]) & 0x7F;
			$chtab = (ord($data[0]) & 0x80) + (((ord($data[1]) & 0xC0) >> 1)) >> 5;
			$sword = ord($data[1]) & 0x3F;
			$dx = ord($data[2]);
			$dy = ord($data[3]);
			$weight = ord($data[4]);
	//		$chtab += $frame_table_chtab;
			if ($image > 0) {
				$sprite = $chtab_bases[$chtab + $frame_table_chtab] + $image - 1;
			} else {
				$sprite = -1;
			}
			print("<tr>");
			print("<td>");
			printf("FRAME %3d -> CHTAB=%d+%d IMAGE=%3d -> SPRITE=%4d\n",$frame,$chtab,$frame_table_chtab,$image,$sprite);
			print("</td>");
			print("<td>");
			printf("<img src='../exp1/%04d.bmp'>",$sprite);
			print("</td>");
			$frame_sprite[$which_table][$frame] = $sprite;
			++$frame;
			print("</tr>");
		}
		print("</table>");
		print("\n");
	}
	return $frame_sprite;
}

function label_addr($x){
	$s=sprintf('$%04X',$x);
	return "<a name='".$s."'>".$s."</a>";
}

function jump_addr($x){
	$s=sprintf('$%04X',$x);
	return "<a href='#".$s."'>".$s."</a>";
}

function parse_seq_table($f,$frame_sprite,$entries){
	print "<h2>Sequence table</h2>";

	//die();
	//$seq_start = 0x91F4 + 0x8000; // ???
	$seq_start = 0xEAE7 + 0x8000;
	$seq_end = 0xFA06 + 0x8000;
	print("<table border='1'>\r\n");
	$which_table = 0;
	$tables = array(
	// 0xEB2B skeleton
	// 0xEBD7
	// 0xEC12 kid
		0xEC29 => 6,
		0xED52 => 0,
		0xf4d0 => 2,
	//	0xf551 => 5,
		0xF570 => 5, // Jaffar closes gate // if Char.action = 0
		0xF582 => 2, // Jaffar closes gate // if Char.frame = $01
		0xf6b0 => 4,
		0xf732 => 3,
		// 0xf862 amazon
		0xF8be => 5,
	);
	for ($p = $seq_start; $p < $seq_end; ) {
		$ad=$p-0x8000;
		$p0=$p;
		if (isset($tables[$ad])) $which_table = $tables[$ad];
		$x = getint1($f,$p);
		switch ($x) {
			case 0xFB: $y=getint1($f,$p); if($y&0x80)$y-=0x100; $line=sprintf("delta X %d",$y); break;
			case 0xFA: $y=getint1($f,$p); if($y&0x80)$y-=0x100; $line=sprintf("delta Y %d",$y); break;
			case 0xFE: $line=("flip"); break;
			case 0xFF: $y=getint2($f,$p); $line=sprintf("jump %s",jump_addr($y)); break;
			case 0xFD: $line=("up"); break;
			case 0xFC: $line=("down"); break;
			case 0xF9: $y=getint1($f,$p); $line=sprintf("action %d",$y); break;
			case 0xF8: $y=getint1($f,$p); $z=getint1($f,$p); if($y&0x80)$y-=0x100; $line=sprintf("set fall %d,%d",$y,$z); break;
			case 0xF7: $y=getint2($f,$p); $line=sprintf("jump if slow-fall %s",jump_addr($y)); break;
			case 0xF6: $line=("nop"); break;
			case 0xF5: $line=("knock up"); break;
			case 0xF4: $line=("knock down"); break;
			case 0xF2: $y=getint1($f,$p); $line=sprintf("sound %d",$y); break;
			case 0xF1: $line=("end level"); break;
			case 0xF3: $y=getint1($f,$p); $line=sprintf("get item %d",$y); break;
			default:
				$sprite = @$frame_sprite[$which_table][$x];
				$line=sprintf("frame %d",$x);
				if ($sprite !== null) $line.=sprintf(" <img src='../exp1/%04d.bmp'>",$sprite);
				break;
		}
		$curr_entries = @$entries[$ad];
		if($curr_entries){
			foreach($curr_entries as $k=>$v) $curr_entries[$k] = sprintf('%d = $%02X',$v,$v);
			$curr_entries = "Sequence ".implode(", ",$curr_entries);
		} else {
			$curr_entries="";
		}
		printf("<tr><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr>\r\n",$curr_entries,label_addr($ad),str2hex(substr($f,$p0,$p-$p0)),$line);
	}
	print("</table>\r\n");
}

$entries = parse_seq_addresses($f);
$frame_sprite = parse_frame_table($f);
//print("!!!!!!!"); print_r($frame_sprite);
parse_seq_table($f,$frame_sprite,$entries);
